import sys
import typing


class MotionPathButtonsPanel:
    bl_label = None
    ''' '''

    bl_region_type = None
    ''' '''

    bl_space_type = None
    ''' '''

    def draw_settings(self, _context, avs, mpath, bones):
        ''' 

        '''
        pass


class MotionPathButtonsPanel_display:
    bl_label = None
    ''' '''

    bl_region_type = None
    ''' '''

    bl_space_type = None
    ''' '''

    def draw_settings(self, _context, avs, mpath, bones):
        ''' 

        '''
        pass



import sys
import typing


class DataPathBuilder:
    data_path = None
    ''' '''

    def resolve(self, real_base, rna_update_from_map, fcurve, log):
        ''' 

        '''
        pass


def anim_data_actions(anim_data):
    ''' 

    '''

    pass


def classes_recursive(base_type, clss):
    ''' 

    '''

    pass


def find_path_new(id_data, data_path, rna_update_from_map, fcurve, log):
    ''' 

    '''

    pass


def id_iter():
    ''' 

    '''

    pass


def update_data_paths(rna_update, log):
    ''' 

    '''

    pass



import sys
import typing


class ProgressReport:
    curr_step = None
    ''' '''

    running = None
    ''' '''

    start_time = None
    ''' '''

    steps = None
    ''' '''

    wm = None
    ''' '''

    def enter_substeps(self, nbr, msg):
        ''' 

        '''
        pass

    def finalize(self):
        ''' 

        '''
        pass

    def initialize(self, wm):
        ''' 

        '''
        pass

    def leave_substeps(self, msg):
        ''' 

        '''
        pass

    def start(self):
        ''' 

        '''
        pass

    def step(self, msg, nbr):
        ''' 

        '''
        pass

    def update(self, msg):
        ''' 

        '''
        pass


class ProgressReportSubstep:
    final_msg = None
    ''' '''

    level = None
    ''' '''

    msg = None
    ''' '''

    nbr = None
    ''' '''

    progress = None
    ''' '''

    def enter_substeps(self, nbr, msg):
        ''' 

        '''
        pass

    def leave_substeps(self, msg):
        ''' 

        '''
        pass

    def step(self, msg, nbr):
        ''' 

        '''
        pass



import sys
import typing
depsgraph_update_post = None
''' on depsgraph update (post)
'''

depsgraph_update_pre = None
''' on depsgraph update (pre)
'''

frame_change_post = None
''' Called after frame change for playback and rendering, after the data has been evaluated for the new frame.
'''

frame_change_pre = None
''' Called after frame change for playback and rendering, before any data is evaluated for the new frame. This makes it possible to change data and relations (for example swap an object to another mesh) for the new frame. Note that this handler is **not** to be used as 'before the frame changes' event. The dependency graph is not available in this handler, as data and relations may have been altered and the dependency graph has not yet been updated for that.
'''

load_factory_preferences_post = None
''' on loading factory preferences (after)
'''

load_factory_startup_post = None
''' on loading factory startup (after)
'''

load_post = None
''' on loading a new blend file (after)
'''

load_pre = None
''' on loading a new blend file (before)
'''

persistent = None
''' Function decorator for callback functions not to be removed when loading new files
'''

redo_post = None
''' on loading a redo step (after)
'''

redo_pre = None
''' on loading a redo step (before)
'''

render_cancel = None
''' on canceling a render job
'''

render_complete = None
''' on completion of render job
'''

render_init = None
''' on initialization of a render job
'''

render_post = None
''' on render (after)
'''

render_pre = None
''' on render (before)
'''

render_stats = None
''' on printing render statistics
'''

render_write = None
''' on writing a render frame (directly after the frame is written)
'''

save_post = None
''' on saving a blend file (after)
'''

save_pre = None
''' on saving a blend file (before)
'''

undo_post = None
''' on loading an undo step (after)
'''

undo_pre = None
''' on loading an undo step (before)
'''

version_update = None
''' on ending the versioning code
'''

xr_session_start_pre = None
''' on starting an xr session (before)
'''



import sys
import typing


def renderer_get() -> str:
    ''' Get GPU to be used for rendering.

    :rtype: str
    :return: GPU name.
    '''

    pass


def vendor_get() -> str:
    ''' Get GPU vendor.

    :rtype: str
    :return: Vendor name.
    '''

    pass


def version_get() -> str:
    ''' Get GPU driver version.

    :rtype: str
    :return: Driver version.
    '''

    pass



import sys
import typing


def code_from_builtin(pygpu_shader_name: str) -> dict:
    ''' Exposes the internal shader code for consultation.

    :param pygpu_shader_name: - 2D_FLAT_COLOR - 2D_IMAGE - 2D_SMOOTH_COLOR - 2D_UNIFORM_COLOR - 3D_FLAT_COLOR - 3D_SMOOTH_COLOR - 3D_UNIFORM_COLOR - 3D_POLYLINE_FLAT_COLOR - 3D_POLYLINE_SMOOTH_COLOR - 3D_POLYLINE_UNIFORM_COLOR
    :type pygpu_shader_name: str
    :rtype: dict
    :return: Vertex, fragment and geometry shader codes.
    '''

    pass


def from_builtin(shader_name: str, config: str = 'DEFAULT'):
    ''' Shaders that are embedded in the blender internal code. They all read the uniform mat4 ModelViewProjectionMatrix , which can be edited by the :mod: gpu.matrix module. You can also choose a shader configuration that uses clip_planes by setting the CLIPPED value to the config parameter. Note that in this case you also need to manually set the value of mat4 ModelMatrix . For more details, you can check the shader code with the :func: gpu.shader.code_from_builtin function.

    :param shader_name: - 2D_FLAT_COLOR - 2D_IMAGE - 2D_SMOOTH_COLOR - 2D_UNIFORM_COLOR - 3D_FLAT_COLOR - 3D_SMOOTH_COLOR - 3D_UNIFORM_COLOR - 3D_POLYLINE_FLAT_COLOR - 3D_POLYLINE_SMOOTH_COLOR - 3D_POLYLINE_UNIFORM_COLOR
    :type shader_name: str
    :param config: - DEFAULT - CLIPPED
    :type config: str
    :return: Shader object corresponding to the given name.
    '''

    pass


def unbind():
    ''' Unbind the bound shader object.

    '''

    pass
